;==============================================================================
; Display object's color in the concole
pro Square::PrintColor

self->GetProperty, color=color
print,'_____________________________'
print,'My color is:   R=',color[0],'    G=',color[1],'    B=',color[2]
print,''

end


;==============================================================================
; Display object's perimeter in the concole
pro Square::PrintPerimeter

print,'_____________________________'
print,'My perimeter is: ',self.perimeter
print,''

end


;==============================================================================
; Display object's area in the concole
pro Square::PrintArea

print,'_____________________________'
print,'My area is: ',self.area
print,''

end


;==============================================================================
; Calculate perimeter given length
pro Square::CalculatePerimeter

self.perimeter = 4*self.length

end


;==============================================================================
; Calculate area given length
; CalculateArea method is defined as both a procedure and a function
; It is bossible to have both!
pro Square::CalculateArea

self->GetProperty, length=length
Self.area = length^2

end

function Square::CalculateArea

self->GetProperty, length=length
Self.area = length^2

return, 1

end


;==============================================================================
; Retrieve object properties
; - This is commonly referred to as an accessor method in OOP
;   since they provide "access" to an object's properties
pro Square::GetProperty, area=area, perimeter=perimeter, length=length,color=color

if (arg_present(area)) then $
    area = self.area

if (arg_present(perimeter)) then $
    perimeter = self.perimeter

if (arg_present(length)) then $
    length = self.length

if (arg_present(color)) then $
    color = self.color

end


;==============================================================================
; Set object properties
; - This is commonly referred to as a mutator method in OOP 
;   since they alter an object's properties.
; - Note: area and perimeter are not set directly - they depend on length property!
pro Square::SetProperty, length=length, color=color

if (n_elements(length) gt 0 && length gt 0.0) then begin
    ; negative values are not allowed
    self.length = length
    self->CalculateArea         ; recalculate area whenever length changes
    self->CalculatePerimeter    ; recalculate perimeter whenever length changes
endif

if (n_elements(color) eq 3) then begin
    ; color must be a 3-element vector
    self.color = color
endif

end

;==============================================================================
; Initialization method.
; Referred to as a Constructor in OOP since it helps construct objects of this class
function Square::Init, _REF_EXTRA=etc

; set default values for object's data members and ensure that they all
; internally consistent!
self.length = 1.0
self.area = 1.0
self.perimeter = 4.0
self.color = [0,0,0]

; In case any additional properties were included in the call to obj_new(),
; pass them on to the setproperty method
if (n_elements(etc) gt 0) then self->setProperty, _EXTRA=etc

return, 1

end


;==============================================================================
; The Destructor
; Called whenever an object of this class is destroyed
pro Square::Cleanup

; no heap variables to free
; or any other tidying up to do
end


;==============================================================================
; Class structure definition
; Defines the data members or variables for an object of this class
pro Square__define

struct = {Square $
          ,area:0.0 $
          ,perimeter:0.0 $
          ,length:0.0 $
          ,color:[0B,0B,0B] $
         }

end
